<?php

namespace Drupal\markdown_exporter\EventSubscriber;

use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerAwareTrait;
use Drupal\Component\Serialization\Json;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Url;
use Markdownify\Parser;
use Markdownify\Converter;

class MarkdownExporterGenerator implements ContainerAwareInterface {

	use ContainerAwareTrait;	
	
	/**
  * Creating content files in MD format
  *
	* Converts the content of the specified types to markdown,
	* and creates a zip containing them.
	* @param array $get_types
	* @param string $mode
  */
	public function bulkExport ($get_types, $mode = '')
	{
		$settings = \Drupal::config('markdown_exporter.settings');
		$md_dir = 'public://' . $settings->get('markdown_exporter_dir');
		$field_config = $settings->get('markdown_exporter_fields');
		// $success = \Drupal::service('file_system')->prepareDirectory(
		// 	$dir,
		// 	 FileSystemInterface::CREATE_DIRECTORY |
		// 		FileSystemInterface::MODIFY_PERMISSIONS);
		// TODO to verify why prepareDirectory() not workign and returning false
		// $success = file_prepare_directory(
		// 	$md_dir,
		// 	 FILE_CREATE_DIRECTORY |
		// 		FILE_MODIFY_PERMISSIONS
		// );
		$success = true;

		\Drupal::service('file_system')->prepareDirectory($md_dir, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);
		// \Drupal::service('file_system')->prepareDirectory('temporary://tmp', FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);

		$mdObj = new Converter(0, MDFY_BODYWIDTH, MDFY_KEEPHTML);
		
		$types = array_values($get_types);
		array_pop($types);
		$query = \Drupal::entityQuery('node');
    $query
      ->condition('type', $types, 'IN')
      ->condition('status', 1)
			->sort('created', 'ASC');

		$nids = $query->execute();
		if (count($nids)) {    
			$node_details = \Drupal\node\Entity\Node::loadMultiple($nids);   
		}
		
		$entity_fields = Json::decode($field_config);		
		
		$files = array();
		if(!empty($node_details)) {
			foreach ($node_details as $node) {
				$url_alias = \Drupal::service('path.alias_manager')
				->getAliasByPath('/node/'.$node->id());
				
				// formatting the alias
				$url_alias_formatted = strpos($url_alias, '/') ?
				 str_replace('/', "-", $url_alias) : $url_alias;
				 
				// Content file in .md format
				$file_name = $url_alias_formatted . '.md';
				$bundle = $node->bundle();
				
				if (
					array_key_exists($bundle, $entity_fields) &&
					 !empty($entity_fields[$bundle])
				) {
					
					$formatter_fields = $entity_fields[$bundle]['formatter'];
					array_unshift($formatter_fields, "type", "title", "created");
					$content_fields = $entity_fields[$bundle]['content'];
					if (!array_key_exists('body', $entity_fields[$bundle]['content'])) {
						$content_fields = array_unshift($content_fields, 'body');
					}
				} else {
					$formatter_fields = array('type', 'title', 'created');
					$content_fields = array('body');
				}
				
				// Preparing MD formatter
				$formatter =  "---\n";
				$formatter .=  "uri:" . $url_alias . "\n";
				$i=0;
				foreach($formatter_fields as $fld1) {
					$field_value = '';
					$formatter_field_data = '';
					
					switch($fld1) {
						case 'type':
							$field_value = $fld1 . ":" . $node->bundle() . "\n";
							break;
						case 'title':
							$field_value = $fld1 . ":" . $node->title->value . "\n";
							break;
						case 'created':
							$field_value = $fld1 . ":" . $node->created->value . "\n";
							break;
						default:
							$customFieldVal = $node->get($fld1)->getValue();
							
							if(isset($customFieldVal)) {
								if (count($customFieldVal) > 1) {
									$field_value .= $fld1 . ":\n";
									foreach($customFieldVal as $valC) {
										foreach($valC as $k=>$v) {
											if (!is_array($v)) {
												$field_value .= "\t" . $k . ":" . $v . "\n";
											}
										}
									}
								} else {
									$formatter_field_data = $node->get($fld1)->getValue();
									if(!is_array($formatter_field_data)) {  
										$field_value = $fld1 . ":" . $formatter_field_data . "\n";
									} else {
										if(isset($formatter_field_data[$node->language][0]['value'])) { 
											$field_value = $fld1 . ":" . $formatter_field_data[$node->language][0]['value'] . "\n";
										} else {
											if(
												isset(
													$formatter_field_data[$node->language][0]['uri']) &&
													 !empty(
														 $formatter_field_data[$node->language][0]['uri']
														)
												){ // for image field
												$field_value = $fld1 . ":" . file_create_url($formatter_field_data[$node->language][0]['uri']) . "\n";
											}
										}
									}
								}								
							}
					}
					$formatter .=  $field_value;
				}
				$formatter .=  "---\n\n";

				// Preparing MD content
				$mdcontent = '';
				if (is_array($content_fields) && !empty($content_fields)) {
					foreach($content_fields as $fld2) {
						if(isset($node->{$fld2})) {
							$content_field_data = $node->{$fld2};
							$field_value = (!is_array($content_field_data)) ?
							 $content_field_data :
							  $content_field_data[$node->language][0]['value'];
							$mdcontent .= $mdObj->parseString($field_value);
						}  
					}
				}

				// Markdown file content
				$contents = $formatter . $mdcontent;

				// and create the file
				// $replace = ($settings->get('markdown_file_overwrite') == TRUE) ?
				//  FileSystemInterface::EXISTS_REPLACE :
				//   FileSystemInterface::EXISTS_RENAME;
				$replace = ($settings->get('markdown_file_overwrite') == TRUE) ?
				FILE_EXISTS_REPLACE : FILE_EXISTS_RENAME;
				if($success) {
					drupal_chmod($md_dir);
					// $base_file = \Drupal::service('file_system')->basename($md_dir . $file_name);
					$base_file = $md_dir . $file_name;
					// $result = file_unmanaged_save_data($contents, $base_file, $replace);
					$result = \Drupal::service('file_system')->saveData($contents, $base_file, FileSystemInterface::EXISTS_REPLACE);
					// $directory = \Drupal::service('file_system')->realpath("public://userslist");
					// \Drupal::service('file_system')->moveUploadedFile($filename, $uri);
					// $result = FileSystemInterface::saveData(
					// 	$contents,
					// 	 $md_dir . '/' . $file_name,
					// 	 $replace
					// );
				}
			}
			$msg = ($mode == 'cli') ? t('MarkDown files are generated & saved in')
			 . ' files/'. variable_get('markdown_exporter_dir', 'markdown_export')
				: $msg = t('MarkDown files are generated!') . ' '
				 . t(
					 '<a href=":url">Click </a> to download the files in ziped format',
					  [':url' => Url::fromRoute('markdown_exporter.download')->toString()]
					);
			
			\Drupal::messenger()->addMessage($msg, 'markdown-export');
		}

		return false;
	}
	
	/**
   * Creating single node content files in MD format
   *
	 * Converts the content of the node to markdown
	 * @param Drupal\Core\Entity\EntityInterface $entity
   */
	public function generateMarkdownNodeContent ($entity) {
		$markdown_exporter_config = \Drupal::config('markdown_exporter.settings');
		$markdown_exporter_fields = Json::decode($markdown_exporter_config->get('markdown_exporter_fields'));
		
		//echo '<pre>'; print_r($markdown_exporter_fields); exit;
		if (!array_key_exists($markdown_exporter_fields, $entity->bundle()) && empty($markdown_exporter_fields[$entity->bundle()])) {
			return;
		}

		$md_dir = 'public://' . $markdown_exporter_config->get('markdown_exporter_dir');
		$success = true;

		\Drupal::service('file_system')->prepareDirectory($md_dir, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);

		$mdObj = new Converter(0, MDFY_BODYWIDTH, MDFY_KEEPHTML);

		$url_alias = \Drupal::service('path.alias_manager')
		->getAliasByPath('/node/'.$entity->id());
		
		// formatting the alias
		$url_alias_formatted = strpos($url_alias, '/') ?
			str_replace('/', "-", $url_alias) : $url_alias;
			
		// Content file in .md format
		$file_name = $url_alias_formatted . '.md';

		$content = $this->_mdContentGenerate($url_alias, $entity, $markdown_exporter_fields);

		$replace = ($markdown_exporter_config->get('markdown_file_overwrite') == TRUE) ?
				FILE_EXISTS_REPLACE : FILE_EXISTS_RENAME;
		if($success) {
			drupal_chmod($md_dir);
			$base_file = $md_dir . $file_name;
			$result = \Drupal::service('file_system')->saveData($contents, $base_file, FileSystemInterface::EXISTS_REPLACE);
		}

		$msg = ($mode == 'cli') ? t('MarkDown files are generated & saved in')
			 . ' files/'. variable_get('markdown_exporter_dir', 'markdown_export')
				: $msg = t('MarkDown files are generated!') . ' '
				 . t(
					 '<a href=":url">Click </a> to download the files in ziped format',
					  [':url' => Url::fromRoute('markdown_exporter.download')->toString()]
					);

		\Drupal::messenger()->addMessage($msg, 'markdown-export');
	}

	/**
   * Private reusuable function
   *
	 * Converts the content of the node to markdown
	 * @param string $url_alias
	 * @param Drupal\Core\Entity\EntityInterface $node
	 * @param array $markdown_exporter_fields
   */
	private function  _mdContentGenerate ($url_alias, $node, $markdown_exporter_fields) {
		$formatter_fields = $markdown_exporter_fields[$node->type]['formatter'];
		$content_fields = $markdown_exporter_fields[$node->type]['content'];
		// Preparing MD formatter
		$formatter =  "---\n";
		$formatter .=  "uri:" . $url_alias . "\n";
		$i=0;
		foreach($formatter_fields as $fld1) {
			$field_value = '';
			$formatter_field_data = '';
			
			switch($fld1) {
				case 'type':
					$field_value = $fld1 . ":" . $node->bundle() . "\n";
					break;
				case 'title':
					$field_value = $fld1 . ":" . $node->title->value . "\n";
					break;
				case 'created':
					$field_value = $fld1 . ":" . $node->created->value . "\n";
					break;
				default:
					$customFieldVal = $node->get($fld1)->getValue();
					
					if(isset($customFieldVal)) {
						if (count($customFieldVal) > 1) {
							$field_value .= $fld1 . ":\n";
							foreach($customFieldVal as $valC) {
								foreach($valC as $k=>$v) {
									if (!is_array($v)) {
										$field_value .= "\t" . $k . ":" . $v . "\n";
									}
								}
							}
						} else {
							$formatter_field_data = $node->get($fld1)->getValue();
							if(!is_array($formatter_field_data)) {  
								$field_value = $fld1 . ":" . $formatter_field_data . "\n";
							} else {
								if(isset($formatter_field_data[$node->language][0]['value'])) { 
									$field_value = $fld1 . ":" . $formatter_field_data[$node->language][0]['value'] . "\n";
								} else {
									if(
										isset(
											$formatter_field_data[$node->language][0]['uri']) &&
											 !empty(
												 $formatter_field_data[$node->language][0]['uri']
												)
										){ // for image field
										$field_value = $fld1 . ":" . file_create_url($formatter_field_data[$node->language][0]['uri']) . "\n";
									}
								}
							}
						}								
					}
			}
			$formatter .=  $field_value;
		}
		$formatter .=  "---\n\n";

		// Preparing MD content
		$mdcontent = '';
		if (is_array($content_fields) && !empty($content_fields)) {
			foreach($content_fields as $fld2) {
				if(isset($node->{$fld2})) {
					$content_field_data = $node->{$fld2};
					$field_value = (!is_array($content_field_data)) ?
					 $content_field_data :
						$content_field_data[$node->language][0]['value'];
					$mdcontent .= $mdObj->parseString($field_value);
				}  
			}
		}

		// Markdown file content
		$contents = $formatter . $mdcontent;

		return $contents;
	}
}