<?php

namespace Drupal\markdown_exporter\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\Process\Process;
use Symfony\Component\Process\Exception\ProcessFailedException;

/**
 * Class MarkdownExport.
 */
class MarkdownExport extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'markdown_exporter_export';
  }
  
  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
		$node_types = \Drupal\node\Entity\NodeType::loadMultiple();
		// If you need to display them in a drop down:
		$content_types = [];
		foreach ($node_types as $node_type) {
			$content_types[$node_type->id()] = $node_type->label();
		}
		$form['types'] = array(
			'#type' => 'checkboxes',
			'#title' => t('Content Types'),
			'#default_value' => array(),
			'#options' => $content_types,
			'#description' => t('Choose content types to be exported.'),
		);
		//submit button
		$form['submit'] = array(
			'#type' => 'submit',
			'#value' => t('Run Export'),
			'#suffix' => t('<hr style="color:#CCC"><p><em style="font-size:0.75rem">**Use custom drush command <strong>"drush mde"</strong> or <strong>"drush markdown-export"</strong> if nodes to be exported are significantly large in number. To export specific content type supply the content type name with the drush command (e.g. drush mde page). For multiple content types, values needs to seperated by single blank space (e.g. drush mde page article)</em></p>'),
		);

    return $form;
  }
  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
		$get_types = $form_state->getValue('types');
		\Drupal::service('markdown_exporter.marekdown_generator')->bulkExport($get_types);
	}
}